@@ -2,9 +2,14 @@
 #define MPU_UTILITY_H
 
 #include <gmp.h>
-#ifndef STANDALONE
 #include "ptypes.h"
-#endif
+
+extern int get_verbose_level(void);
+extern void set_verbose_level(int level);
+
+extern gmp_randstate_t* get_randstate(void);
+extern void init_randstate(unsigned long seed);
+extern void clear_randstate(void);
 
 /* tdiv_r is faster, but we'd need to guarantee the input is positive */
 #define mpz_mulmod(r, a, b, n, t)  \
@@ -41,7 +46,7 @@ extern void polyz_root_deg1(mpz_t root, mpz_t* pn, mpz_t NMOD);
 extern void polyz_root_deg2(mpz_t root1, mpz_t root2, mpz_t* pn, mpz_t NMOD);
 
 /* Find roots of a polynomial, slightly modified. */
-extern void polyz_roots_modp(mpz_t** roots, long *nroots,
+extern void polyz_roots_modp(mpz_t** roots, long *nroots, long maxroots,
                              mpz_t *pP, long dP, mpz_t NMOD,
                              gmp_randstate_t* p_randstate);
 
